uab.modules.LayoutManager = uab:NewModule("LayoutManager", "AceModuleCore-2.0")
uab.modules.LayoutManager:SetModuleMixins("AceDebug-2.0", "AceModuleCore-2.0")

uabLayoutManager = uab.modules.LayoutManager
uabLayoutManager.core = uab
uabLayoutManager.layouts = {}

function uabLayoutManager:OnInitialize()
	self.super.OnInitialize(self)

    self.core.options.args.lookandfeel.args.layout = {
        type = 'group',
        name = "Layout",
        desc = "Select and configure button layout options",
		order = 10,
        args = {
            configure = {
                type = 'group',
                name = "Configure",
                desc = "Configure current layout options",
                order = 10,
                args = {}
            },
            select = {
                type = 'text',
                name = "Select",
                usage = "<name>",
                desc = "Select the current layout to use",
                order = 20,
                get = function()
                	local mgr = self:Get(uab.db.profile.layout)
                	return mgr:GetDisplayName()
				end,
                set = function(name)
                	uab.db.profile.layout = self:GetByDisplayName(name).name
                	self:LayoutChanged()
                end,
                validate = self.layouts
            },
        },
    }
end

function uabLayoutManager:OnEnable()
	self.super.OnEnable(self)
    self:LayoutChanged()
end

function uabLayoutManager:GetByDisplayName(displayname)
    for name, module in self:IterateModules() do
        if module:GetDisplayName() == displayname then
            if module:IsEnabled() then
                return module
            end
        end
    end
end

function uabLayoutManager:Get(mgrname)
    for name, module in self:IterateModules() do
        if name == mgrname then
            if module:IsEnabled() then
                return module
            end
        end
    end
end

function uabLayoutManager:LayoutSettingsChanged(contextName, activator)
	local mgr = self:Get(uab.db.profile.layout)
	local layout = mgr:GetLayout(contextName or "main")

    if not activator then
    	uab.main:IterateActivators("SetLayout", nil, layout)
    else
        activator:SetLayout(nil, layout)
    end
end

function uabLayoutManager:LayoutChanged()
	local mgr = self:Get(uab.db.profile.layout)
	
	local layout = mgr:GetLayout("main")
	local config = layout:GetConfig()
	
	uab.options.args.lookandfeel.args.layout.args.configure.args = config:MenuOptionArgs()
	
	self:LayoutSettingsChanged("main")
end

--[[
LayoutManager module prototype

When implementing a LayoutManager module:
 - variables to set
 	- self.defaultDB to define your own default settings for all layouts, or
          to hold any settings you want to use.  Be sure to include the following
          settings as they are used to control the use of a LayoutManager:
 	    - disabled
 - OnInitialize functionality to implement
    - set self.defaultDB here if needed
    - call super.OnInitialize(self)
    - set any other variables you may need and perform any additional first-time
	initialization
 - Methods to implement
    - OnEnable (optional)
    - OnDisable (optional)
    - GetLayout(contextName) - required
        - Used to allocate a new layout in the specified context.  The settings
          to use are maintained by you related to the specified context.  If you
          have no settings for that context, you must initialize the returned layout
          to use default settings appropriate for that layout.
    - GetDisplayName() - required
        - Used to retrieve the name to appear in the FuBar drop-down menu for this
          layout manager.
--]]

uabLayoutManager.modulePrototype.core = uabLayoutManager
uabLayoutManager.modulePrototype.controls = {}
uabLayoutManager.modulePrototype.layoutManagerDefaultDB = {
    disabled = nil,
}

function uabLayoutManager.modulePrototype:OnInitialize()
    if not self.db then
        self.core.core:RegisterDefaults(self.name, "profile", self.defaultDB or self.layoutManagerDefaultDB)
        self.db = self.core.core:AcquireDBNamespace(self.name)
    end

    self.debugging = self.db.profile.debug
    if self:IsEnabled() then
	    table.insert(self.core.layouts, self:GetDisplayName())
	end
end

function uabLayoutManager.modulePrototype:IsEnabled()
    return not self.db.profile.disabled
end
